## ----setup, include=FALSE---------------------------------------------------------------------------------------------
library(knitr)
knitr::opts_chunk$set(dpi = 100, echo= TRUE, warning=FALSE, message=FALSE, 
                      fig.show=TRUE, fig.keep = 'all', out.width = '90%')


## ----echo = FALSE-----------------------------------------------------------------------------------------------------
library(mixOmics) # import the mixOmics library

data(srbct) # extract the small round bull cell tumour data
X <- srbct$gene # use the gene expression data as the X matrix
Y <- srbct$class # use the class data as the Y matrix - NON permuted
Yp <- sample(srbct$class) # use the class data as the Y matrix - permuted


## ----echo = FALSE-----------------------------------------------------------------------------------------------------
optimal.ncomp <- 3
optimal.keepX <- c(9, 260, 30)


## ---------------------------------------------------------------------------------------------------------------------
optimal.ncomp
optimal.keepX


## ----echo = FALSE-----------------------------------------------------------------------------------------------------
final.splsda.normal <- splsda(X, Y, 
                       ncomp = optimal.ncomp, 
                       keepX = optimal.keepX)

final.splsda.permuted <- splsda(X, Yp, 
                       ncomp = optimal.ncomp, 
                       keepX = optimal.keepX)


## ----echo = FALSE-----------------------------------------------------------------------------------------------------
# form new perf() objects which utilises the final models
perf.splsda.srbct.normal <- perf(final.splsda.normal, 
                          folds = 5, nrepeat = 10, # use repeated cross-validation
                          validation = "Mfold", dist = "max.dist",  # use max.dist measure
                          progressBar = FALSE)

perf.splsda.srbct.permuted <- perf(final.splsda.permuted, 
                          folds = 5, nrepeat = 10, # use repeated cross-validation
                          validation = "Mfold", dist = "max.dist",  # use max.dist measure
                          progressBar = FALSE)


## ----echo = FALSE, collapse = TRUE------------------------------------------------------------------------------------
stab.normal.c1 <- length(perf.splsda.srbct.normal$features$stable[[1]][perf.splsda.srbct.normal$features$stable[[1]] > 0.4])
stab.permuted.c1 <- length(perf.splsda.srbct.permuted$features$stable[[1]][perf.splsda.srbct.permuted$features$stable[[1]] > 0.4])

stab.normal.c2 <- length(perf.splsda.srbct.normal$features$stable[[2]][perf.splsda.srbct.normal$features$stable[[2]] > 0.4])
stab.permuted.c2 <- length(perf.splsda.srbct.permuted$features$stable[[2]][perf.splsda.srbct.permuted$features$stable[[2]] > 0.4])

stab.normal.c3 <- length(perf.splsda.srbct.normal$features$stable[[3]][perf.splsda.srbct.normal$features$stable[[3]] > 0.4])
stab.permuted.c3 <- length(perf.splsda.srbct.permuted$features$stable[[3]][perf.splsda.srbct.permuted$features$stable[[3]] > 0.4])

print("Normal component 1")
cat(" - Features with high stability: ", stab.normal.c1)
cat(" - Mean stability: ", mean(perf.splsda.srbct.normal$features$stable[[1]]))

print("Normal component 2")
cat(" - Features with high stability: ", stab.normal.c2)
cat(" - Mean stability: ", mean(perf.splsda.srbct.normal$features$stable[[2]]))

print("Normal component 3")
cat(" - Features with high stability: ", stab.normal.c3)
cat(" - Mean stability: ", mean(perf.splsda.srbct.normal$features$stable[[3]]))

print("Permutted component 1")
cat(" - Features with high stability: ", stab.permuted.c1)
cat(" - Mean stability: ", mean(perf.splsda.srbct.permuted$features$stable[[1]]))

print("Permutted component 2")
cat(" - Features with high stability: ", stab.permuted.c2)
cat(" - Mean stability: ", mean(perf.splsda.srbct.permuted$features$stable[[2]]))

print("Permutted component 3")
cat(" - Features with high stability: ", stab.permuted.c3)
cat(" - Mean stability: ", mean(perf.splsda.srbct.permuted$features$stable[[3]]))


## ----fig.show = "hold", out.width = "49%", echo = FALSE---------------------------------------------------------------
plotIndiv(final.splsda.normal, comp = c(1,2), # plot samples from final model
          group = srbct$class, ind.names = FALSE, # colour by class label
          ellipse = TRUE, legend = TRUE, # include 95% confidence ellipse
          title = ' (a) sPLS-DA on SRBCT, comp 1 & 2')

plotIndiv(final.splsda.normal, comp = c(1,3), # plot samples from final model
          group = srbct$class, ind.names = FALSE,  # colour by class label
          ellipse = TRUE, legend = TRUE, # include 95% confidence ellipse
          title = '(b) sPLS-DA on SRBCT, comp 1 & 3')


## ----fig.show = "hold", out.width = "49%", echo = FALSE---------------------------------------------------------------
plotIndiv(final.splsda.permuted, comp = c(1,2), # plot samples from final model
          group = srbct$class, ind.names = FALSE, # colour by class label
          ellipse = TRUE, legend = TRUE, # include 95% confidence ellipse
          title = ' (a) sPLS-DA on SRBCT, comp 1 & 2')

plotIndiv(final.splsda.permuted, comp = c(1,3), # plot samples from final model
          group = srbct$class, ind.names = FALSE,  # colour by class label
          ellipse = TRUE, legend = TRUE, # include 95% confidence ellipse
          title = '(b) sPLS-DA on SRBCT, comp 1 & 3')


## ----fig.align = "center", echo = FALSE-------------------------------------------------------------------------------
var.name.short <- substr(srbct$gene.name[, 2], 1, 10) #  simplified gene names

plotVar(final.splsda.normal, comp = c(1,2), var.names = list(var.name.short), 
        cex = 3) # generate correlation circle plot


## ----fig.align = "center", echo = FALSE-------------------------------------------------------------------------------
var.name.short <- substr(srbct$gene.name[, 2], 1, 10) #  simplified gene names

plotVar(final.splsda.permuted, comp = c(1,2), var.names = list(var.name.short), 
        cex = 3) # generate correlation circle plot


## ----echo = FALSE, collapse = TRUE------------------------------------------------------------------------------------
ProduceAveragedPerfMetrics <- function(X, Y, folds, bootstraps) {
  metrics <- data.frame(matrix(0, nrow = 4, ncol = length(unique(Y))))
  colnames(metrics) <- unique(Y)
  rownames(metrics) <- c("error", "precision", "recall", "f1")
  
  for (B in 1:bootstraps) {
    
    testingProportion <- floor((1/folds)*nrow(X))
    notTestedIdx <- 1:nrow(X)
    
    for (fold in 1:folds) {
      test <- sample(notTestedIdx, testingProportion) # randomly select samples in testing
      train <- setdiff(1:nrow(X), test) # rest is part of the train set
      
      # store matrices into training and test set:
      X.train <- X[train, ]
      X.test <- X[test,]
      Y.train <- Y[train]
      Y.test <- Y[test]
      
      train.splsda <- splsda(X.train, Y.train, ncomp = optimal.ncomp, keepX = optimal.keepX)
      predict.splsda <- predict(train.splsda, X.test, dist = "centroids.dist")
      
      preds <- predict.splsda$class$centroids.dist[,optimal.ncomp]
      conf.matrix <- table(factor(preds, levels = levels(Y)), Y.test)
      
      for (class in unique(Y)) {
        
        class = as.character(class)
        df <- data.frame(conf.matrix)
        
        tp <- conf.matrix[class, class]
        
        tn.df <- df[which(as.character(df$Var1)!=class),]
        tn.df <- tn.df[which(as.character(tn.df$Y.test)!=class),]
        tn <- sum(tn.df$Freq)
        
        fn.df <- df[which(as.character(df$Y.test) == class), ]
        fn.df <- fn.df[which(as.character(fn.df$Var1) != class),]
        fn <- sum(fn.df$Freq)
        
        fp.df <- df[which(as.character(df$Var1) == class), ]
        fp.df <- fp.df[which(as.character(fp.df$Y.test) != class),]
        fp <- sum(fp.df$Freq)
        
        error <- (fp+fn)/(tp+tn+fp+fn)
        precision <- tp/(tp+fp)
        recall <- tp/(tp+fn)
        f1 <- (2*precision*recall)/(precision+recall)
        
        if (is.na(precision)) {
          precision = 0
        }
        if (is.na(recall)) {
          recall = 0
        }
        if (is.na(f1)) {
          f1 = 0
        }
        
        
        metrics[,class] <- metrics[,class] + c(error/folds, precision/folds, recall/folds, f1/folds)
      }
      
      notTestedIdx <- setdiff(notTestedIdx, test)
    }
    
    
  }
  
  metrics["error",] <- metrics["error",]/bootstraps
  metrics["precision",] <- metrics["precision",]/bootstraps
  metrics["recall",] <- metrics["recall",]/bootstraps
  metrics["f1",] <- metrics["f1",]/bootstraps
  return(metrics)
}

normal.metrics <- ProduceAveragedPerfMetrics(X, Y, 5, 100)
permuted.metrics <- ProduceAveragedPerfMetrics(X, Yp, 5, 100)

print("Normal model performance metrics")
normal.metrics

print("Permuted model performance metrics")
permuted.metrics

